@@ -22,6 +22,7 @@
 #include <string.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 
@@ -240,29 +241,16 @@ media_art_checksum_for_data (GChecksumType  checksum_type,
  * @artist: (allow-none): the artist
  * @title: (allow-none): the title
  * @prefix: (allow-none): the prefix for cache files, for example "album"
- * @file: (allow-none): a #GFile representing the actual media art or %NULL
  * @cache_file: (out) (transfer full) (allow-none): a pointer to a
  * #GFile which represents the cached file for media art, or %NULL
  * a #GFile representing the user&apos;s cache path, or %NULL
- * @local_file: (out) (transfer full) (allow-none): a pointer to a
  * #GFile representing the location of the local media art
  *
  * Gets the files pointing to cache files suitable for storing the media
  * art provided by the @artist, @title and @file arguments. @cache_file
- * will point to a location in the XDG user cache directory, meanwhile
- * @local_file will point to a cache file that resides in the same
- * filesystem than @file.
+ * will point to a location in the XDG user cache directory..
  *
- * The @file provided is required if @local_file is to be returned.
- * The @local_file relates to a location on the media the local file
- * system or media storage it is found on, so for example, if you have
- * a mounted volume with MP3s on it in
- * <filename>file:///media/martyn/pendrive</filename>, the @local_file
- * will point to a URI that looks like
- * <filename>file:///media/martyn/pendrive/.mediaartlocal/...</filename>.
- *
- * The @cache_file is very different to the @local_file, the
- * @cache_file relates to a symlink stored in XDG cache directories
+ * The @cache_file relates to a symlink stored in XDG cache directories
  * for the user. A @cache_file would be expected to look like
  * <filename>file:///home/martyn/.cache/media-art/...</filename>. This
  * is normally the location that is most useful (assuming the cache
@@ -271,7 +259,10 @@ media_art_checksum_for_data (GChecksumType  checksum_type,
  * When done, both #GFile<!-- -->s must be freed with g_object_unref() if
  * non-%NULL.
  *
- * Returns: %TRUE if @cache_file or @local_file were returned, otherwise %FALSE.
+ * This operation should not use i/o, but it depends on the backend
+ * GFile implementation.
+ *
+ * Returns: %TRUE if @cache_file was returned, otherwise %FALSE.
  *
  * Since: 0.2.0
  */
@@ -279,9 +270,7 @@ gboolean
 media_art_get_file (const gchar  *artist,
                     const gchar  *title,
                     const gchar  *prefix,
-		    GFile        *file,
-		    GFile       **cache_file,
-		    GFile       **local_file)
+                    GFile       **cache_file)
 {
 	const gchar *space_checksum = "7215ee9c7d9dc229d2921a40e899ec5f";
 	const gchar *a, *b;
@@ -299,16 +288,12 @@ media_art_get_file (const gchar  *artist,
 		*cache_file = NULL;
 	}
 
-	if (local_file) {
-		*local_file = NULL;
-	}
-
 	/* Rules:
 	 * 1. artist OR title must be non-NULL.
-	 * 2. file AND local_file must be non-NULL OR cache_file must be non-NULL
+	 * 2. cache_file must be non-NULL
 	 */
 	g_return_val_if_fail (artist != NULL || title != NULL, FALSE);
-	g_return_val_if_fail (((!G_IS_FILE (file) && !G_IS_FILE (local_file)) || !G_IS_FILE (cache_file)), FALSE);
+	g_return_val_if_fail (!G_IS_FILE (cache_file), FALSE);
 
 	if (artist) {
 		artist_stripped = media_art_strip_invalid_entities (artist);
@@ -332,10 +317,6 @@ media_art_get_file (const gchar  *artist,
 	                        "media-art",
 	                        NULL);
 
-	if (!g_file_test (dir, G_FILE_TEST_EXISTS)) {
-		g_mkdir_with_parents (dir, 0770);
-	}
-
 	if (artist) {
 		a = artist_checksum;
 		b = title ? title_checksum : space_checksum;
@@ -366,19 +347,6 @@ media_art_get_file (const gchar  *artist,
 		g_free (filename);
 	}
 
-	if (local_file) {
-		GFile *parent;
-
-		parent = g_file_get_parent (file);
-		if (parent) {
-			filename = g_build_filename (".mediaartlocal", art_filename, NULL);
-			*local_file = g_file_resolve_relative_path (parent, filename);
-			g_free (filename);
-
-			g_object_unref (parent);
-		}
-	}
-
 	g_free (dir);
 	g_free (art_filename);
 
@@ -390,22 +358,18 @@ media_art_get_file (const gchar  *artist,
  * @artist: (allow-none): the artist
  * @title: (allow-none): the title
  * @prefix: (allow-none): the prefix, for example "album"
- * @uri: (allow-none): the uri of the file or %NULL
  * @cache_path: (out) (transfer full) (allow-none): a string
  * representing the path to the cache for this media art
  * path or %NULL
- * @local_uri: (out) (transfer full) (allow-none): a string
- * representing the URI to the local media art or %NULL
  *
  * This function calls media_art_get_file() by creating a #GFile for
  * @uri and passing the same arguments to media_art_get_file(). For more
  * details about what this function does, see media_art_get_file().
  *
  * Get the path to media art for a given resource. Newly allocated
- * data returned in @cache_path and @local_uri must be freed with g_free().
+ * data returned in @cache_path must be freed with g_free().
  *
- * Returns: %TRUE if @cache_path or @local_uri were returned,
- * otherwise %FALSE.
+ * Returns: %TRUE if @cache_path was returned, otherwise %FALSE.
  *
  * Since: 0.2.0
  */
@@ -413,94 +377,64 @@ gboolean
 media_art_get_path (const gchar  *artist,
                     const gchar  *title,
                     const gchar  *prefix,
-                    const gchar  *uri,
-                    gchar       **cache_path,
-                    gchar       **local_uri)
+                    gchar       **cache_path)
 {
-	GFile *file = NULL, *cache_file = NULL, *local_file = NULL;
+	GFile *cache_file = NULL;
 
 	/* Rules:
 	 * 1. artist OR title must be non-NULL.
-	 * 2. file AND local_file must be non-NULL OR cache_file must be non-NULL
+	 * 2. cache_file must be non-NULL
 	 */
 	g_return_val_if_fail (artist != NULL || title != NULL, FALSE);
-	g_return_val_if_fail ((uri != NULL && local_uri != NULL) || cache_path != NULL, FALSE);
+	g_return_val_if_fail (cache_path != NULL, FALSE);
 
-	if (uri) {
-		file = g_file_new_for_uri (uri);
-	}
-
-	media_art_get_file (artist, title, prefix, file,
-			    cache_path ? &cache_file : NULL,
-			    local_uri ? &local_file : NULL);
+	media_art_get_file (artist, title, prefix, cache_path ? &cache_file : NULL);
 	if (cache_path) {
 		*cache_path = cache_file ? g_file_get_path (cache_file) : NULL;
 	}
 
-	if (local_uri) {
-		*local_uri = local_file ? g_file_get_uri (local_file) : NULL;
-	}
-
-	if (file) {
-		g_object_unref (file);
-	}
-
 	return TRUE;
 }
 
-static void
-media_art_remove_foreach (gpointer data,
-                          gpointer user_data)
-{
-	gchar *filename = data;
-	gboolean total_success = * (gboolean *) user_data;
-	gboolean success;
-
-	success = g_unlink (filename) == 0;
-	total_success &= success;
-
-	if (!success) {
-		g_warning ("Could not delete file '%s'", filename);
-	}
-
-	g_free (filename);
-}
-
 /**
  * media_art_remove:
  * @artist: artist the media art belongs to
  * @album: (allow-none): album the media art belongs or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
+ * @error: location to store the error occurring, or %NULL to ignore
  *
  * Removes media art for given album/artist provided.
  *
- * Returns: #TRUE on success, otherwise #FALSE.
+ * If @artist and @album are %NULL, ALL media art cache is removed.
+ *
+ * Returns: #TRUE on success, otherwise #FALSE where @error will be set.
  *
  * Since: 0.2.0
  */
 gboolean
-media_art_remove (const gchar *artist,
-                  const gchar *album)
+media_art_remove (const gchar   *artist,
+                  const gchar   *album,
+                  GCancellable  *cancellable,
+                  GError       **error)
 {
-	GError *error = NULL;
-	GHashTable *table = NULL;
+	GError *local_error = NULL;
 	const gchar *name;
 	GDir *dir;
 	gchar *dirname;
-	GList *to_remove = NULL;
-	gchar *target = NULL;
 	gboolean success = TRUE;
 
 	g_return_val_if_fail (artist != NULL && artist[0] != '\0', FALSE);
 
 	dirname = g_build_filename (g_get_user_cache_dir (), "media-art", NULL);
 
-	dir = g_dir_open (dirname, 0, &error);
-	if (!dir || error) {
+	dir = g_dir_open (dirname, 0, &local_error);
+	if (!dir || local_error) {
 		/* Nothing to do if there is no directory in the first place. */
 		g_debug ("Removing media-art for artist:'%s', album:'%s': directory could not be opened, %s",
-		         artist, album, error ? error->message : "no error given");
+		         artist, album, local_error ? local_error->message : "no error given");
+
+		g_clear_error (&local_error);
 
-		g_clear_error (&error);
 		if (dir) {
 			g_dir_close (dir);
 		}
@@ -512,51 +446,205 @@ media_art_remove (const gchar *artist,
 		return TRUE;
 	}
 
-	table = g_hash_table_new_full (g_str_hash,
-	                               g_str_equal,
-	                               (GDestroyNotify) g_free,
-	                               (GDestroyNotify) NULL);
+	/* NOTE: We expect to not find some of these paths for
+	 * artist/album conbinations, so don't error in those
+	 * cases...
+	 */
+	if (artist || album) {
+		gchar *target = NULL;
+		gint removed = 0;
 
-	/* The get_path API does stripping itself */
-	media_art_get_path (artist, album, "album", NULL, &target, NULL);
-	if (target) {
-		g_hash_table_replace (table, target, target);
-	}
+		/* The get_path API does stripping itself */
+		media_art_get_path (artist, album, "album", &target);
 
-	/* Add the album path also (to which the symlinks are made) */
-	if (album) {
-		media_art_get_path (NULL, album, "album", NULL, &target, NULL);
 		if (target) {
-			g_hash_table_replace (table, target, target);
+			if (g_unlink (target) != 0) {
+				g_debug ("Could not delete file '%s'", target);
+			} else {
+				g_message ("Removed media-art for artist:'%s', album:'%s': deleting file '%s'",
+				           artist, album, target);
+				removed++;
+			}
+
+			g_free (target);
+		}
+
+		/* Add the album path also (to which the symlinks are made) */
+		if (album) {
+			media_art_get_path (NULL, album, "album", &target);
+			if (target) {
+				if (g_unlink (target) != 0) {
+					g_debug ("Could not delete file '%s'", target);
+				} else {
+					g_message ("Removed media-art for album:'%s': deleting file '%s'",
+					           album, target);
+					removed++;
+				}
+
+				g_free (target);
+			}
+		}
+
+		success = removed > 0;
+	} else {
+		for (name = g_dir_read_name (dir);
+		     name != NULL;
+		     name = g_dir_read_name (dir)) {
+			gchar *target;
+
+			target = g_build_filename (dirname, name, NULL);
+
+			if (g_unlink (target) != 0) {
+				g_warning ("Could not delete file '%s'", target);
+				success = FALSE;
+			} else {
+				g_message ("Removing all media-art: deleted file '%s'", target);
+			}
+
+			g_free (target);
 		}
 	}
 
-	/* Perhaps we should have an internal list of media art files that we made,
-	 * instead of going over all the media art (which could also have been made
-	 * by other softwares) */
-	for (name = g_dir_read_name (dir); name != NULL; name = g_dir_read_name (dir)) {
-		gpointer value;
-		gchar *full;
-
-		full = g_build_filename (dirname, name, NULL);
-		value = g_hash_table_lookup (table, full);
-
-		if (!value) {
-			g_message ("Removing media-art for artist:'%s', album:'%s': deleting file '%s'",
-			           artist, album, name);
-			to_remove = g_list_prepend (to_remove, (gpointer) full);
-		} else {
-			g_free (full);
-		}
+	if (!success) {
+		g_set_error_literal (error,
+		                     G_IO_ERROR,
+		                     G_IO_ERROR_FAILED,
+		                     _("Could not remove one or more files from media art cache"));
 	}
 
-	g_list_foreach (to_remove, media_art_remove_foreach, &success);
-	g_list_free (to_remove);
-
-	g_hash_table_unref (table);
-
 	g_dir_close (dir);
 	g_free (dirname);
 
 	return success;
 }
+
+typedef struct {
+	gchar *artist;
+	gchar *album;
+} RemoveData;
+
+static RemoveData *
+remove_data_new (const gchar *artist,
+                 const gchar *album)
+{
+	RemoveData *data;
+
+	data = g_slice_new0 (RemoveData);
+	data->artist = g_strdup (artist);
+	data->album = g_strdup (album);
+
+	return data;
+}
+
+static void
+remove_data_free (RemoveData *data)
+{
+	if (!data) {
+		return;
+	}
+
+	g_free (data->artist);
+	g_free (data->album);
+	g_slice_free (RemoveData, data);
+}
+
+static void
+remove_thread (GTask        *task,
+               gpointer      source_object,
+               gpointer      task_data,
+               GCancellable *cancellable)
+{
+	RemoveData *data = task_data;
+	GError *error = NULL;
+	gboolean success = FALSE;
+
+	if (!g_cancellable_set_error_if_cancelled (cancellable, &error)) {
+		success = media_art_remove (data->artist,
+		                            data->album,
+		                            cancellable,
+		                            &error);
+	}
+
+	if (error) {
+		g_task_return_error (task, error);
+	} else {
+		g_task_return_boolean (task, success);
+	}
+}
+
+/**
+ * media_art_remove_async:
+ * @artist: artist the media art belongs to
+ * @album: (allow-none): album the media art belongs or %NULL
+ * @source_object: (allow-none): the #GObject this task belongs to,
+ * can be %NULL.
+ * @io_priority: the [I/O priority][io-priority] of the request
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to
+ * ignore
+ * @callback: (scope async): a #GAsyncReadyCallback to call when the
+ * request is satisfied
+ * @user_data: (closure): the data to pass to callback function
+ *
+ * Removes media art for given album/artist provided. Precisely the
+ * same operation as media_art_remove() is performing, but
+ * asynchronously.
+ *
+ * When all i/o for the operation is finished the @callback will be
+ * called.
+ *
+ * In case of a partial error the callback will be called with any
+ * succeeding items and no error, and on the next request the error
+ * will be reported. If a request is cancelled the callback will be
+ * called with %G_IO_ERROR_CANCELLED.
+ *
+ * During an async request no other sync and async calls are allowed,
+ * and will result in %G_IO_ERROR_PENDING errors.
+ *
+ * Any outstanding i/o request with higher priority (lower numerical
+ * value) will be executed before an outstanding request with lower
+ * priority. Default priority is %G_PRIORITY_DEFAULT.
+ *
+ * Since: 0.7.0
+ */
+void
+media_art_remove_async (const gchar           *artist,
+                        const gchar           *album,
+                        gint                   io_priority,
+                        GObject               *source_object,
+                        GCancellable          *cancellable,
+                        GAsyncReadyCallback    callback,
+                        gpointer               user_data)
+{
+	GTask *task;
+
+	task = g_task_new (source_object, cancellable, callback, user_data);
+	g_task_set_task_data (task, remove_data_new (artist, album), (GDestroyNotify) remove_data_free);
+	g_task_set_priority (task, io_priority);
+	g_task_run_in_thread (task, remove_thread);
+	g_object_unref (task);
+}
+
+/**
+ * media_art_remove_finish:
+ * @source_object: (allow-none): the #GObject this task belongs to,
+ * can be %NULL.
+ * @result: a #GAsyncResult.
+ * @error: a #GError location to store the error occurring, or %NULL
+ * to ignore.
+ *
+ * Finishes the asynchronous operation started with
+ * media_art_remove_async().
+ *
+ * Returns: %TRUE on success, otherwise %FALSE when @error will be set.
+ *
+ * Since: 0.7.0
+ **/
+gboolean
+media_art_remove_finish (GObject       *source_object,
+                         GAsyncResult  *result,
+                         GError       **error)
+{
+	g_return_val_if_fail (g_task_is_valid (result, source_object), FALSE);
+
+	return g_task_propagate_boolean (G_TASK (result), error);
+}
