@@ -1,5 +1,7 @@
 #include "marshal.h"
-#include <glib-object.h>
+
+#include	<glib-object.h>
+
 
 #ifdef G_ENABLE_DEBUG
 #define g_marshal_value_peek_boolean(v)  g_value_get_boolean (v)
@@ -47,6 +49,7 @@
 #define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
 #endif /* !G_ENABLE_DEBUG */
 
+
 /* VOID:STRING,STRING (marshal.list:1) */
 void
 media_art_marshal_VOID__STRING_STRING (GClosure     *closure,
@@ -56,13 +59,13 @@ media_art_marshal_VOID__STRING_STRING (GClosure     *closure,
                                        gpointer      invocation_hint G_GNUC_UNUSED,
                                        gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer data1,
-                                                    gpointer arg1,
-                                                    gpointer arg2,
-                                                    gpointer data2);
-  GCClosure *cc = (GCClosure *) closure;
-  gpointer data1, data2;
+  typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer     data1,
+                                                    gpointer     arg_1,
+                                                    gpointer     arg_2,
+                                                    gpointer     data2);
   GMarshalFunc_VOID__STRING_STRING callback;
+  GCClosure *cc = (GCClosure*) closure;
+  gpointer data1, data2;
 
   g_return_if_fail (n_param_values == 3);
 
@@ -93,16 +96,16 @@ media_art_marshal_VOID__STRING_STRING_STRING_BOOLEAN_BOOLEAN (GClosure     *clos
                                                               gpointer      invocation_hint G_GNUC_UNUSED,
                                                               gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_STRING_STRING_BOOLEAN_BOOLEAN) (gpointer data1,
-                                                                           gpointer arg1,
-                                                                           gpointer arg2,
-                                                                           gpointer arg3,
-                                                                           gboolean arg4,
-                                                                           gboolean arg5,
-                                                                           gpointer data2);
-  GCClosure *cc = (GCClosure *) closure;
-  gpointer data1, data2;
+  typedef void (*GMarshalFunc_VOID__STRING_STRING_STRING_BOOLEAN_BOOLEAN) (gpointer     data1,
+                                                                           gpointer     arg_1,
+                                                                           gpointer     arg_2,
+                                                                           gpointer     arg_3,
+                                                                           gboolean     arg_4,
+                                                                           gboolean     arg_5,
+                                                                           gpointer     data2);
   GMarshalFunc_VOID__STRING_STRING_STRING_BOOLEAN_BOOLEAN callback;
+  GCClosure *cc = (GCClosure*) closure;
+  gpointer data1, data2;
 
   g_return_if_fail (n_param_values == 6);
 
